@@ -3,8 +3,15 @@ use strict;
 use warnings;
 use DBI;
 use DBD::SQLite;
+use version;
 
-our $VERSION = '0.06';
+our $VERSION = '0.08';
+
+our $VERSION_INT =
+  1_000_000 * version->parse($VERSION)->numify;
+
+# Archive::Zip crc32 of 'Graph::Feather'
+our $APPLICATION_ID = -801604570;
 
 sub new {
   my ($class, %options) = @_;
@@ -22,6 +29,11 @@ sub new {
   ###################################################################
   # Deploy DB schema
 
+  $self->{dbh}->do(sprintf q{
+    PRAGMA application_id = %i;
+    PRAGMA user_version = %u;
+  }, $APPLICATION_ID, $VERSION_INT);
+
   $self->{dbh}->do(q{
 
     -----------------------------------------------------------------
@@ -230,10 +242,24 @@ sub delete_vertices {
 
 sub delete_edge {
   my ($self, $src, $dst) = @_;
-  delete_edges($self, [$src, $dst]);
+  feather_delete_edges($self, [$src, $dst]);
 }
 
 sub delete_edges {
+  my ($self, @vertices) = @_;
+  my @edges;
+
+  while (@vertices) {
+    my ($src, $dst) = splice @vertices, 0, 2;
+    warn 'delete_edges takes vertex list, not list of vertex pairs'
+      if ref $src; 
+    push @edges, [ $src, $dst ];
+  }
+
+  $self->feather_delete_edges(@edges);
+}
+
+sub feather_delete_edges {
   my ($self, @edges) = @_;
   
   delete_edge_attributes($self, @$_) for @edges;
@@ -819,6 +845,8 @@ sub _copy_vertices_edges_attributes {
   # copy edges
   $rhs->add_edges($lhs->edges);
 
+  # TODO(bh): use new plural methods instead
+
   # copy graph attributes
   for my $n ($lhs->get_graph_attribute_names) {
     $rhs->set_graph_attribute($n,
@@ -853,6 +881,50 @@ sub feather_import_from {
   _copy_vertices_edges_attributes($source, $self);
 }
 
+sub _feather_restore_from_file {
+  my ($self, $path) = @_;
+
+  $self->{dbh}->begin_work();
+
+  $self->{dbh}->sqlite_backup_from_file($path);
+
+  my $sth1 = $self->_prepare(q{
+    PRAGMA application_id
+  });
+
+  my ($application_id) = $self->{dbh}->selectrow_array($sth1);
+
+  my $sth2 = $self->_prepare(q{
+    PRAGMA user_version
+  });
+
+  my ($user_version) = $self->{dbh}->selectrow_array($sth2);
+
+  die "$path was not created by this version of Graph::Feather"
+    unless $application_id eq $APPLICATION_ID
+      and $user_version eq $VERSION_INT;
+
+  $self->{dbh}->commit();
+
+  # The following is needed because attribute values have a
+  # dual life to support Perl references as attribute values.
+
+  $self->set_graph_attribute(@$_) for
+    $self->{dbh}->selectall_array(q{
+      SELECT attribute_name, attribute_value
+      FROM Graph_Attribute});
+
+  $self->set_vertex_attribute(@$_) for
+    $self->{dbh}->selectall_array(q{
+      SELECT vertex, attribute_name, attribute_value
+      FROM Vertex_Attribute});
+
+  $self->set_edge_attribute(@$_) for
+    $self->{dbh}->selectall_array(q{
+      SELECT src, dst, attribute_name, attribute_value
+      FROM Edge_Attribute});
+}
+
 1;
 
 __END__
@@ -1016,6 +1088,12 @@ to the target graph, overwriting any existing attributes.
 Adds vertices, edges, their attributes, and any graph attributes
 from the other graph, overwriting any existing attributes.
 
+=item feather_delete_edges(@edges)
+
+In the original Graph module, C<delete_edges> takes a flat list of
+vertices instead of a list of vertex pairs. This function takes a
+list of vertex pairs instead.
+
 =back
 
 =head1 TODO
